@@ -16,3 +16,15 @@ class NotFound(BioLibError):
 
 class DockerContainerNotFoundDuringExecutionException(Exception):
     pass
+
+
+class RetryLimitException(Exception):
+    pass
+
+
+class StorageDownloadFailed(Exception):
+    pass
+
+
+class CloudJobFinishedError(Exception):
+    pass
