@@ -1,19 +1,24 @@
+import json
 import tempfile
 import time
 import tarfile
 import zipfile
 import os
 import io
+import re
 import shlex
+from datetime import datetime
 
 import docker  # type: ignore
+import docker.types  # type: ignore
+
 from docker.errors import ImageNotFound, APIError  # type: ignore
 from docker.models.containers import Container  # type: ignore
 
 from biolib import utils
 from biolib.biolib_binary_format import ModuleOutput, ModuleInput
 from biolib.biolib_docker_client import BiolibDockerClient
-from biolib.biolib_errors import DockerContainerNotFoundDuringExecutionException
+from biolib.biolib_errors import DockerContainerNotFoundDuringExecutionException, BioLibError
 from biolib.biolib_logging import logger, logger_no_user_data
 from biolib.compute_node import utils as compute_node_utils
 from biolib.compute_node.job_worker.docker_image_cache import DockerImageCache
@@ -77,33 +82,55 @@ class DockerExecutor(BaseExecutor):
         self._runtime_tar_path = f'{user_data_tar_dir}/runtime_{self._random_docker_id}.tar'
         self._input_tar_path = f'{user_data_tar_dir}/input_{self._random_docker_id}.tar'
 
+        if utils.IS_RUNNING_IN_CLOUD and not utils.BIOLIB_SECRETS_TMPFS_PATH:
+            error_message = 'Running in cloud but no TMPFS path has been set for secrets'
+            logger_no_user_data.error(error_message)
+            raise BioLibError(error_message)
+
+        # If BIOLIB_SECRETS_TMPFS_PATH is set create the temporary directory there
+        self._tmp_secrets_dir = tempfile.TemporaryDirectory(dir=utils.BIOLIB_SECRETS_TMPFS_PATH or None)
+        os.chmod(self._tmp_secrets_dir.name, 0o755)
+
     def execute_module(self, module_input_serialized: bytes) -> bytes:
-        send_status_update = self._options['send_status_update']
-        send_system_exception = self._options['send_system_exception']
-
-        module_input = ModuleInput(module_input_serialized).deserialize()
-
-        # TODO: fix these status updates such that they also make sense for run-dev
-        send_status_update(StatusUpdate(progress=55, log_message='Pulling images...'))
-
-        self._pull()
-
-        send_status_update(StatusUpdate(progress=70, log_message='Computing...'))
-        start_time = time.time()
-
-        stdout, stderr, exit_code, mapped_output_files = self._execute_helper(module_input)
-
         try:
-            module_output_serialized: bytes = ModuleOutput().serialize(stdout, stderr, exit_code, mapped_output_files)
-            logger.debug(f'Compute time: {time.time() - start_time}')
-            return module_output_serialized
+            job_uuid = self._options['job']['public_id']
+            send_status_update = self._options['send_status_update']
+            send_system_exception = self._options['send_system_exception']
 
-        except Exception as exception:
-            raise ComputeProcessException(
-                exception,
-                SystemExceptionCodes.FAILED_TO_SERIALIZE_AND_SEND_MODULE_OUTPUT.value,
-                send_system_exception
-            ) from exception
+            module_input = ModuleInput(module_input_serialized).deserialize()
+
+            # TODO: fix these status updates such that they also make sense for run-dev
+            send_status_update(StatusUpdate(progress=55, log_message='Pulling images...'))
+
+            self._pull()
+
+            send_status_update(StatusUpdate(progress=70, log_message='Computing...'))
+            start_time = time.time()
+
+            logger_no_user_data.debug(f'Starting execution of {job_uuid}.')
+            try:
+                stdout, stderr, exit_code, mapped_output_files = self._execute_helper(module_input)
+            except docker.errors.NotFound as docker_error:
+                raise DockerContainerNotFoundDuringExecutionException from docker_error
+            except Exception as exception:
+                raise ComputeProcessException(
+                    exception,
+                    SystemExceptionCodes.FAILED_TO_RUN_COMPUTE_CONTAINER.value,
+                    self._send_system_exception
+                ) from exception
+            logger_no_user_data.debug(f'Completed execution of {job_uuid}.')
+
+            try:
+                module_output_serialized = ModuleOutput().serialize(stdout, stderr, exit_code, mapped_output_files)
+                logger.debug(f'Compute time: {time.time() - start_time}')
+                return module_output_serialized
+
+            except Exception as exception:
+                raise ComputeProcessException(
+                    exception,
+                    SystemExceptionCodes.FAILED_TO_SERIALIZE_AND_SEND_MODULE_OUTPUT.value,
+                    send_system_exception
+                ) from exception
         finally:
             try:
                 self.cleanup()
@@ -137,59 +164,52 @@ class DockerExecutor(BaseExecutor):
             ) from exception
 
     def _execute_helper(self, module_input):
+        job_uuid = self._options['job']['public_id']
+        logger_no_user_data.debug(f'Initializing container for {job_uuid}.')
         self._initialize_docker_container(module_input)
 
         if self._options['runtime_zip_bytes']:
             self._map_and_copy_runtime_files_to_container(self._options['runtime_zip_bytes'], module_input['arguments'])
 
+        logger_no_user_data.debug(f'_map_and_copy_input_files_to_container for {job_uuid}.')
         self._map_and_copy_input_files_to_container(module_input['files'], module_input['arguments'])
 
-        try:
-            docker_api_client = BiolibDockerClient.get_docker_client().api
-            logger.debug('Starting Docker container')
+        docker_api_client = BiolibDockerClient.get_docker_client().api
+        logger_no_user_data.debug(f'Starting Docker container for {job_uuid}')
 
-            stdout_and_stderr_stream = docker_api_client.attach(
-                container=self._docker_container.id,
-                stderr=True,
-                stdout=True,
-                stream=True,
+        stdout_and_stderr_stream = docker_api_client.attach(
+            container=self._docker_container.id,
+            stderr=True,
+            stdout=True,
+            stream=True,
+        )
+
+        self._docker_container.start()
+
+        if self._options['job']['app_version'].get('stdout_render_type') != 'markdown':
+            for stdout_and_stderr in stdout_and_stderr_stream:
+                # Default messages to empty bytestring instead of None
+                stdout_and_stderr = stdout_and_stderr if stdout_and_stderr is not None else b''
+
+                self._send_stdout_and_stderr(stdout_and_stderr)
+
+        exit_code = docker_api_client.wait(self._docker_container.id)['StatusCode']
+        # 137 is the error code from linux OOM killer (Should catch 90% of OOM errors)
+        if exit_code == 137:
+            raise ComputeProcessException(
+                MemoryError(),
+                SystemExceptionCodes.OUT_OF_MEMORY.value,
+                self._send_system_exception
             )
 
-            self._docker_container.start()
+        logger_no_user_data.debug(f'Docker container exited with code {exit_code} for {job_uuid}')
 
-            if self._options['job']['app_version'].get('stdout_render_type') != 'markdown':
-                for stdout_and_stderr in stdout_and_stderr_stream:
-                    # Default messages to empty bytestring instead of None
-                    stdout_and_stderr = stdout_and_stderr if stdout_and_stderr is not None else b''
+        full_stdout = docker_api_client.logs(self._docker_container.id, stdout=True, stderr=False)
+        full_stderr = docker_api_client.logs(self._docker_container.id, stdout=False, stderr=True)
 
-                    self._send_stdout_and_stderr(stdout_and_stderr)
+        mapped_output_files = self._get_output_files(arguments=module_input['arguments'])
+        return full_stdout, full_stderr, exit_code, mapped_output_files
 
-            exit_code = docker_api_client.wait(self._docker_container.id)['StatusCode']
-            # 137 is the error code from linux OOM killer (Should catch 90% of OOM errors)
-            if exit_code == 137:
-                raise ComputeProcessException(
-                    MemoryError(),
-                    SystemExceptionCodes.OUT_OF_MEMORY.value,
-                    self._send_system_exception
-                )
-
-            logger.debug(f'Docker container exited with code {exit_code}')
-
-            full_stdout = docker_api_client.logs(self._docker_container.id, stdout=True, stderr=False)
-            full_stderr = docker_api_client.logs(self._docker_container.id, stdout=False, stderr=True)
-
-            mapped_output_files = self._get_output_files(arguments=module_input['arguments'])
-            return full_stdout, full_stderr, exit_code, mapped_output_files
-
-        except docker.errors.NotFound as docker_error:
-            raise DockerContainerNotFoundDuringExecutionException from docker_error
-
-        except Exception as exception:
-            raise ComputeProcessException(
-                exception,
-                SystemExceptionCodes.FAILED_TO_RUN_COMPUTE_CONTAINER.value,
-                self._send_system_exception
-            ) from exception
 
     def cleanup(self):
         # Don't clean up if already in the process of doing so, or done doing so
@@ -215,6 +235,7 @@ class DockerExecutor(BaseExecutor):
             )
 
         logger.debug(f"Deleted compute container in: {time.time() - container_time}")
+        self._tmp_secrets_dir.cleanup()
 
     # TODO: type this method
     def _initialize_docker_container(self, module_input):
@@ -227,7 +248,36 @@ class DockerExecutor(BaseExecutor):
             internal_network = self._options['internal_network']
             extra_hosts: Dict[str, str] = {}
             dns_list: List[str] = []
-            environment_vars = module.get('secrets', {})
+
+            biolib_system_secret = {
+                'version': '1.0.0',
+                'job_uuid': self._options['job']['public_id'],
+                'job_auth_token': self._options['job']['auth_token'],
+            }
+            secrets: Dict[str, str] = dict(
+                **module.get('secrets', {}),
+                biolib_system_secret=json.dumps(biolib_system_secret, indent=4),
+            )
+            docker_volume_mounts.append(self._create_secrets_mount(secrets))
+
+            environment_vars = {}
+
+            # Include secrets and job info as env vars for app versions created before 20/11/2022
+            app_version_created_at = datetime.strptime(
+                self._options['job']['app_version']['created_at'],
+                '%Y-%m-%dT%H:%M:%S.%fZ',
+            )
+            if app_version_created_at < datetime(2022, 11, 30, 0, 0):
+                environment_vars = module.get('secrets', {})
+                environment_vars.update({
+                    'BIOLIB_JOB_UUID': self._options['job']['public_id'],
+                    'BIOLIB_JOB_AUTH_TOKEN': self._options['job']['auth_token']
+                })
+
+            if utils.IS_RUNNING_IN_CLOUD and self._options['cloud_job']:
+                environment_vars.update({
+                    'BIOLIB_JOB_MAX_RUNTIME_IN_SECONDS': self._options['cloud_job']['max_runtime_in_seconds'],
+                })
 
             for proxy in self._options['remote_host_proxies']:
                 proxy_ip = proxy.get_ip_address_on_network(internal_network)
@@ -236,7 +286,9 @@ class DockerExecutor(BaseExecutor):
                     environment_vars.update({
                         'BIOLIB_BASE_URL': f'http://{proxy_ip}',
                         'BIOLIB_CLOUD_BASE_URL': f'http://{proxy_ip}',
+                        # This should be removed eventually, but will break apps calling apps on older versions
                         'BIOLIB_CLOUD_RESULTS_BASE_URL': f'http://{proxy_ip}',
+                        'BIOLIB_CLOUD_JOB_STORAGE_BASE_URL': f'http://{proxy_ip}',
                         # Inform container if we are targeting public biolib as we change the BIOLIB_BASE_URL
                         'BIOLIB_ENVIRONMENT_IS_PUBLIC_BIOLIB': bool(utils.BASE_URL_IS_PUBLIC_BIOLIB)
                     })
@@ -248,23 +300,28 @@ class DockerExecutor(BaseExecutor):
                 dns_proxy_ip_address: str = container_networks[internal_network.name]['IPAddress']
                 dns_list = [dns_proxy_ip_address]
 
-            if self._options['job'].get('arguments_override_command'):
-                # In this case, arguments contains a user specified command to run in the app
-                command = module_input['arguments']
-            else:
-                command = shlex.split(module['command']) + module_input['arguments']
-
             create_container_args = {
-                'command': command,
                 'environment': environment_vars,
                 'extra_hosts': extra_hosts,
                 'image': self._image_uri,
                 'mounts': docker_volume_mounts,
                 'network': internal_network.name,
                 'working_dir': module['working_directory'],
-                'dns': dns_list
+                'dns': dns_list,
             }
 
+            if self._options['job'].get('arguments_override_command'):
+                # In this case, arguments contains a user specified command to run in the app
+                create_container_args.update({
+                    'command': module_input['arguments'],
+                    'entrypoint': ''
+                })
+
+            else:
+                create_container_args.update({
+                    'command': shlex.split(module['command']) + module_input['arguments']
+                })
+
             app_version = self._options['job']['app_version']
             if app_version.get('main_output_file') or app_version.get('stdout_render_type') == 'text':
                 create_container_args['tty'] = True
@@ -457,3 +514,20 @@ class DockerExecutor(BaseExecutor):
             ) from exception
 
         return mapped_output_files
+
+    def _create_secrets_mount(self, secrets: Dict[str, str]) -> docker.types.Mount:
+        job_uuid = self._options['job']['public_id']
+
+        for key, value in secrets.items():
+            if re.match(r'^[a-zA-Z0-9-_]+$', key):
+                with open(f'{self._tmp_secrets_dir.name}/{key}', 'w') as secret_file:
+                    secret_file.write(value)
+            else:
+                logger_no_user_data.warning(f'Job {job_uuid} uses secret with a key not matching validation regex')
+
+        return docker.types.Mount(
+            read_only=True,
+            source=f'{self._tmp_secrets_dir.name}/',
+            target='/biolib/secrets/',
+            type='bind',
+        )
