@@ -6,7 +6,7 @@ from biolib.compute_node.webserver.webserver_types import ComputeNodeInfo
 from biolib.typing_utils import TypedDict, Callable, Optional, List, Dict
 from biolib.compute_node.remote_host_proxy import RemoteHostProxy
 from biolib.biolib_api_client.app_types import Module
-from biolib.biolib_api_client.job_types import Job, CloudJob
+from biolib.biolib_api_client.job_types import CreatedJobDict, CloudJob
 
 
 class StatusUpdate(TypedDict):
@@ -16,7 +16,8 @@ class StatusUpdate(TypedDict):
 
 class RemoteExecuteOptions(TypedDict):
     biolib_base_url: str
-    job: Job
+    job: CreatedJobDict
+    result_name_prefix: Optional[str]
     root_job_id: str
 
 
@@ -30,7 +31,7 @@ class LocalExecutorOptions(TypedDict):
     biolib_base_url: str
     compute_node_info: Optional[ComputeNodeInfo]
     internal_network: Optional[Network]
-    job: Job
+    job: CreatedJobDict
     cloud_job: Optional[CloudJob]
     large_file_systems: Dict[str, LargeFileSystem]
     module: Module
