@@ -17,27 +17,30 @@ from docker.models.networks import Network  # type: ignore
 from docker.models.containers import Container  # type: ignore
 
 from biolib.biolib_binary_format.stdout_and_stderr import StdoutAndStderr
+from biolib.compute_node.job_worker.job_legacy_input_wait_timeout_thread import JobLegacyInputWaitTimeout
+from biolib.compute_node.job_worker.job_storage import JobStorage
 from biolib.compute_node.job_worker.large_file_system import LargeFileSystem
-from biolib.biolib_errors import DockerContainerNotFoundDuringExecutionException
+from biolib.biolib_errors import DockerContainerNotFoundDuringExecutionException, BioLibError, \
+    StorageDownloadFailed
 from biolib.compute_node.job_worker.job_max_runtime_timer_thread import JobMaxRuntimeTimerThread
 from biolib.compute_node.remote_host_proxy import RemoteHostProxy
 from biolib.typing_utils import Optional, List, Dict
 from biolib import utils
-from biolib.biolib_api_client import ModuleEnvironment, Job, JobWrapper, Module, AppVersionOnJob, BiolibApiClient, \
-    RemoteHost
+from biolib.biolib_api_client import ModuleEnvironment, CreatedJobDict, JobWrapper, Module, AppVersionOnJob, \
+    BiolibApiClient, RemoteHost
 from biolib.biolib_api_client.biolib_job_api import BiolibJobApi
 from biolib.biolib_docker_client import BiolibDockerClient
-from biolib.biolib_singularity_client import BiolibSingularityClient
 from biolib.biolib_logging import logger, logger_no_user_data
-from biolib.compute_node.job_worker.executors import DockerExecutor, RemoteExecutor, SingularityExecutor
+from biolib.compute_node.job_worker.executors import DockerExecutor
 from biolib.compute_node.job_worker.executors.base_executor import BaseExecutor
-from biolib.compute_node.job_worker.executors.types import LocalExecutorOptions, StatusUpdate, RemoteExecuteOptions
+from biolib.compute_node.job_worker.executors.types import LocalExecutorOptions, StatusUpdate
 from biolib.compute_node.socker_listener_thread import SocketListenerThread
 from biolib.compute_node.socket_sender_thread import SocketSenderThread
 from biolib.compute_node.job_worker.mappings import Mappings, path_without_first_folder
 from biolib.compute_node.job_worker.utils import ComputeProcessException, log_disk_and_memory_usage_info
 from biolib.compute_node.utils import get_package_type, SystemExceptionCodes, SystemExceptionCodeMap
-from biolib.biolib_binary_format import SavedJob, SystemStatusUpdate, ModuleInput, SystemException
+from biolib.biolib_binary_format import SavedJob, SystemStatusUpdate, ModuleInput, SystemException, \
+    UnencryptedModuleOutput
 
 _DNS_PROXY_IMAGE_URI = 'spx01/blocky@sha256:485ef739233341d3e14a64db646754aba7322c34645f804c3c0c00556d9a2cd1'
 DEFAULT_BUFFER_SIZE = 1024
@@ -72,9 +75,10 @@ class _JobWorker:
             self._socket_port = socket_port
             self._received_messages_queue: Queue = Queue()
             self._messages_to_send_queue: Queue = Queue()
+            self._legacy_input_wait_timeout_thread: Optional[JobLegacyInputWaitTimeout] = None
 
             self._app_version_id_to_runtime_zip: Dict[str, bytes] = {}
-            self._jobs: Dict[str, Job] = {}
+            self._jobs: Dict[str, CreatedJobDict] = {}
             self._root_job_wrapper: Optional[JobWrapper] = None
 
             self._dns_proxy: Optional[Container] = None
@@ -98,8 +102,9 @@ class _JobWorker:
         self._connect_to_parent()
 
     def _handle_exit_gracefully(self, signum: int, frame: FrameType) -> None:  # pylint: disable=unused-argument
+        job_id = self._root_job_wrapper["job"]["public_id"] if self._root_job_wrapper else None
         logger_no_user_data.debug(
-            f'_JobWorker got exit signal {signal.Signals(signum).name}'  # pylint: disable=no-member
+            f'_JobWorker ({job_id}) got exit signal {signal.Signals(signum).name}'  # pylint: disable=no-member
         )
         self._received_messages_queue.put(self._STOP_HANDLE_MESSAGE_LOOP)
         self._cleanup()
@@ -116,7 +121,8 @@ class _JobWorker:
                 if package_type == 'SavedJob':
                     self._handle_save_job_wrapper(package)
                     if utils.IS_RUNNING_IN_CLOUD:
-                        job_uuid = self._root_job_wrapper['job']['public_id']
+                        job = self._root_job_wrapper['job']
+                        job_uuid = job['public_id']
                         max_runtime_in_seconds = self._root_job_wrapper['cloud_job']['max_runtime_in_seconds']
                         logger_no_user_data.debug(
                             f'Job "{job_uuid}" will have max run time set to {max_runtime_in_seconds} seconds'
@@ -126,15 +132,40 @@ class _JobWorker:
                             max_runtime_in_seconds=max_runtime_in_seconds,
                         ).start()
 
+                        try:
+                            module_input_serialized = JobStorage.get_module_input(job=job)
+                        except StorageDownloadFailed:
+                            # Expect module input to be handled in a separate ModuleInput package
+                            self._legacy_input_wait_timeout_thread = JobLegacyInputWaitTimeout(
+                                input_max_wait_in_seconds=120,
+                                job_uuid=job_uuid,
+                                send_system_exception=self.send_system_exception,
+                            )
+                            self._legacy_input_wait_timeout_thread.start()
+                            continue
+                        except Exception as error:
+                            raise error
+
+                        try:
+                            self._run_root_job(module_input_serialized)
+
+                        # This error occurs when trying to access the container after the job worker has cleaned it up.
+                        # In that case stop the computation.
+                        except DockerContainerNotFoundDuringExecutionException as err:
+                            if self.is_cleaning_up:
+                                break
+                            else:
+                                raise err
+
                 elif package_type == 'ModuleInput':
                     if not self._root_job_wrapper:
                         raise Exception('No job saved yet')
 
+                    if self._legacy_input_wait_timeout_thread:
+                        self._legacy_input_wait_timeout_thread.stop()
+
                     try:
-                        module_output_serialized = self._run_job(
-                            job=self._root_job_wrapper['job'],
-                            module_input_serialized=package,
-                        )
+                        self._run_root_job(package)
 
                     # This error occurs when trying to access the container after the job worker has cleaned it up.
                     # In that case stop the computation.
@@ -144,19 +175,6 @@ class _JobWorker:
                         else:
                             raise err
 
-                    module_output_size = len(module_output_serialized)
-                    job_uuid = self._root_job_wrapper['job']['public_id']
-                    logger_no_user_data.debug(
-                        f'Job "{job_uuid}" finished computation with module output size '
-                        f'of {module_output_size} bytes'
-                    )
-
-                    module_output_file_path = os.path.join(self.job_temporary_dir, "module_output.bbf")
-                    with open(module_output_file_path, 'wb') as module_output_file:
-                        module_output_file.write(module_output_serialized)
-
-                    self._send_status_update(StatusUpdate(progress=94, log_message='Computation finished'))
-
                 else:
                     logger_no_user_data.error('Package type from parent was not recognized')
 
@@ -167,7 +185,7 @@ class _JobWorker:
             except Exception as exception:
                 raise ComputeProcessException(
                     exception,
-                    SystemExceptionCodes.UNKOWN_COMPUTE_PROCESS_ERROR.value,
+                    SystemExceptionCodes.UNKNOWN_COMPUTE_PROCESS_ERROR.value,
                     self.send_system_exception
                 ) from exception
 
@@ -238,19 +256,16 @@ class _JobWorker:
 
         # TODO: start downloading runtime zip already at this point
 
-    def _start_network_and_remote_host_proxies(self, job: Job) -> None:
+    def _start_network_and_remote_host_proxies(self, job: CreatedJobDict) -> None:
         app_version = job['app_version']
         job_id = job['public_id']
         remote_hosts = app_version['remote_hosts']
-
-        for module in app_version['modules']:
-            if module['environment'] == 'biolib-app' and module['name'] != 'main':
-                remote_hosts.append(
-                    {
-                        'hostname': 'AppCallerProxy',
-                    },
-                )
-                break
+        if utils.IS_RUNNING_IN_CLOUD:
+            remote_hosts.append(
+                {
+                    'hostname': 'AppCallerProxy',
+                },
+            )
 
         docker_client = BiolibDockerClient.get_docker_client()
         try:
@@ -388,12 +403,17 @@ port: 53
 
         self._dns_proxy.reload()
 
-    def _run_app_version(self, app_version_id: str, module_input_serialized: bytes, caller_job: Job) -> bytes:
-        job: Job = BiolibJobApi.create(app_version_id, caller_job=caller_job['public_id'])
+    def _run_app_version(
+        self,
+        app_version_id: str,
+        module_input_serialized: bytes,
+        caller_job: CreatedJobDict
+    ) -> bytes:
+        job: CreatedJobDict = BiolibJobApi.create(app_version_id, caller_job=caller_job['public_id'])
         self._jobs[job['public_id']] = job
         return self._run_job(job, module_input_serialized)
 
-    def _run_job(self, job: Job, module_input_serialized: bytes) -> bytes:
+    def _run_job(self, job: CreatedJobDict, module_input_serialized: bytes) -> bytes:
         job_uuid = job['public_id']
         logger_no_user_data.info(f'Job "{job_uuid}" running...')
         if self._root_job_wrapper is None:
@@ -404,7 +424,6 @@ port: 53
             root_job = self._jobs[root_job['caller_job']]
 
         root_job_id = root_job['public_id']
-        modules = job['app_version'].get('modules')
         if job.get('arguments_override_command') and not job['app_version']['app']['allow_client_side_execution']:
             raise ComputeProcessException(
                 Exception("Command override not allowed"),
@@ -412,16 +431,12 @@ port: 53
                 self.send_system_exception
             )
 
-        if modules is None or utils.BIOLIB_FORCE_REMOTE_COMPUTE or (
-                not BiolibDockerClient.is_docker_running() and not BiolibSingularityClient.is_singularity_running()
-        ):
-            return RemoteExecutor.execute_job(
-                RemoteExecuteOptions(
-                    biolib_base_url=self._root_job_wrapper['BASE_URL'],
-                    job=job,
-                    root_job_id=root_job_id,
-                ),
-                module_input_serialized,
+        modules = job['app_version'].get('modules')
+        if not modules:
+            raise ComputeProcessException(
+                Exception("No modules found on job"),
+                SystemExceptionCodes.NO_MODULES_FOUND_ON_JOB.value,
+                self.send_system_exception
             )
 
         main_module = self._get_module_from_name(modules, module_name='main')
@@ -496,10 +511,6 @@ port: 53
         elif module['environment'] == ModuleEnvironment.BIOLIB_ECR.value and BiolibDockerClient.is_docker_running():
             executor_instance = DockerExecutor(options)
 
-        elif module['environment'] == ModuleEnvironment.BIOLIB_ECR.value and \
-                BiolibSingularityClient.is_singularity_running():
-            executor_instance = SingularityExecutor(options)
-
         else:
             raise Exception(f"Unsupported module environment {module['environment']}")
 
@@ -637,3 +648,36 @@ port: 53
                 self.send_system_exception,
                 may_contain_user_data=False
             ) from exception
+
+    def _run_root_job(self, module_input_serialized: bytes) -> None:
+        # Make typechecker happy
+        if not self._root_job_wrapper or not self.job_temporary_dir:
+            raise BioLibError('Undefined job_wrapper or job_temporary_dir')
+
+        module_output_serialized = self._run_job(
+            job=self._root_job_wrapper['job'],
+            module_input_serialized=module_input_serialized,
+        )
+
+        module_output_size = len(module_output_serialized)
+        job_uuid = self._root_job_wrapper['job']['public_id']
+        logger_no_user_data.debug(
+            f'Job "{job_uuid}" finished computation with module output size '
+            f'of {module_output_size} bytes'
+        )
+
+        # TODO: Remove when webserver endpoint /v1/job/<job_id>/result/ is deprecated
+        legacy_module_output_file_path = os.path.join(self.job_temporary_dir, JobStorage.legacy_module_output_file_name)
+        with open(legacy_module_output_file_path, mode='wb') as file:
+            file.write(module_output_serialized)
+
+        module_output_file_path = os.path.join(self.job_temporary_dir, JobStorage.module_output_file_name)
+        with open(legacy_module_output_file_path, 'rb') as legacy_module_output, \
+                open(module_output_file_path, 'wb') as module_output:
+
+            UnencryptedModuleOutput.write_to_file_from_legacy_module_output_file(
+                input_file=legacy_module_output,
+                output_file=module_output,
+            )
+
+        self._send_status_update(StatusUpdate(progress=94, log_message='Computation finished'))
